/**
 * textbar.js - scripts for TextBar Drupal module.
 * Version 0.01
 * By Michael Geary, http://mg.to/
 * Adapted from the DokuWiki toolbar (http://wiki.splitbrain.org/wiki:dokuwiki), which was
 * adapted from the MediaWiki toolbar (http://wikipedia.sourceforge.net/), which was
 * adapted from the phpBB toolbar (http://www.phpbb.com/).
 */


/**
 * Browser detection
 */
var textbar_clientPC  = navigator.userAgent.toLowerCase(); // Get client info
var textbar_is_gecko  = ((textbar_clientPC.indexOf('gecko')!=-1) && (textbar_clientPC.indexOf('spoofer')==-1)
                && (textbar_clientPC.indexOf('khtml') == -1) && (textbar_clientPC.indexOf('netscape/7.0')==-1));
var textbar_is_safari = ((textbar_clientPC.indexOf('AppleWebKit')!=-1) && (textbar_clientPC.indexOf('spoofer')==-1));
var textbar_is_khtml  = (navigator.vendor == 'KDE' || ( document.childNodes && !document.all && !navigator.taintEnabled ));
if (textbar_clientPC.indexOf('opera')!=-1) {
    var textbar_is_opera = true;
    var textbar_is_opera_preseven = (window.opera && !document.childNodes);
    var textbar_is_opera_seven = (window.opera && document.childNodes);
}


/**
 * Escape some special characters
 */
function textbar_escapeQuotes( text )
{
    return text.replace( /'/g, "\\'" ).replace( /"/g, '&quot;' ).replace( /\n/g, "\\n" );
}


/**
 * Format the toolbar buttons
 */
function textbar_formatButtons( id, format )
{
    var buttons =
    [
        [ 'simple', 'b',  'bold',    'Bold Text'                    ],
        [ 'simple', 'i',  'italic',  'Italic Text'                  ],
        [ 'simple', 'c',  'code',    'Code Text'                    ],
        [ 'simple', '1',  'fonth1',  'Level 1 Headline'             ],
        [ 'simple', '2',  'fonth2',  'Level 2 Headline'             ],
        [ 'simple', '3',  'fonth3',  'Level 3 Headline'             ],
        [ 'simple', '4',  'fonth4',  'Level 4 Headline'             ],
        [ 'simple', '5',  'fonth5',  'Level 5 Headline'             ],

        [ 'linker', 'l',  'link',    'link text', 'Internal Link'   ],
        [ 'linker', null, 'extlink', 'link text', 'External Link'   ],

        [ 'simple', null, 'list',    'Ordered List Item'            ],
        [ 'simple', null, 'list_ul', 'Unordered List Item'          ],

        [ 'insert', null, 'rule',    'Horizontal Rule'              ]

//      textbar_mediaButton( id, 'images/image.png', 'Add Images and other files', 'm', '' ) +
    ];

    var html = '<div style="xfloat: left; margin: 0px; xmargin-right: 8px;" />';

    for( var iButton = 0;  iButton < buttons.length;  iButton++ )
    {
        var button = buttons[iButton];

        var type = button[0];
        var accessKey = button[1];
        var name = button[2];
        var sampleText = button[3];
        var speedTip = button[4];

        var imageFile = 'images/' + name + '.png';
        sampleText = textbar_escapeQuotes( sampleText );
        speedTip = ( speedTip ? textbar_escapeQuotes( speedTip ) : sampleText );

        var accessAttr = '';
        if( accessKey )
        {
            accessAttr = "accesskey=\"" + accessKey + "\" ";
            speedTip = speedTip + ' [Alt+' + accessKey.toUpperCase() + ']';
        }

        html +=
            "<a " + accessAttr +
            "href=\"javascript:textbar_clickButton('" +
            id + "','" + type + "','" + name + "','" + sampleText + "');\">" +
            "<img width=\"24\" height=\"24\" src=\"" +
            "modules/textbar/" + imageFile + "\" border=\"0\" alt=\"" +
            speedTip + "\" title=\"" + speedTip + "\"></a>";
    }

    html += '</div>';

    return html;
}


/**
 * Obsolete: This adds a button for the MediaSelection Popup
 */
/*
function textbar_mediaButton( imageFile, speedTip, accessKey, namespace )
{
    speedTip = textbar_escapeQuotes( speedTip );
    document.write( "<a " );
    if( accessKey )
        document.write( "accesskey=\"" + accessKey + "\" " );

    document.write( "href=\"javascript:void(window.open('" + baseURL + "media.php?ns=" +
        namespace + "','mediaselect','width=600,height=320,left=70,top=50,scrollbars=yes'));\">" );
    document.write( "<img width=\"24\" height=\"24\" src=\"" +
        baseURL + imageFile + "\" border=\"0\" alt=\"" +
        speedTip + "\" title=\"" + speedTip + "\">" );
    document.write("</a>");
}
*/


/**
 * Handle a click on a toolbar button
 */
function textbar_clickButton( id, type, code, sampleText )
{
    var format = textbar_getInputFormat();
    if( ! format.tagset )
    {
        alert( 'Unsupported input format' );
        return;
    }

    var tags = format.tagset[code];

    var actions =
    {
        insert: function()
        {
            textbar_insertText( id, tags[0] );
        },

        linker: function()
        {
            textbar_insertTags( id, tags[0], tags[1], sampleText );
        },

        simple: function()
        {
            textbar_insertTags( id, tags[0], tags[1], sampleText );
        }
    }

    actions[type]();
}


/**
 * apply tagOpen/tagClose to selection in textarea, use sampleText instead
 * of selection if there is none.
 *
 * @author phpBB development team
 * @author MediaWiki development team
 * @author Andreas Gohr <andi@splitbrain.org>
 * @author Jim Raynor <jim_raynor@web.de>
 */
function textbar_insertTags( id, tagOpen, tagClose, sampleText )
{
    tagClose = tagClose || '';

    var txtarea = document.getElementById(id);

    if( document.selection  &&  ! textbar_is_gecko )  // IE
    {
        var theSelection = document.selection.createRange().text;
        var replaced = true;

        if( ! theSelection )
        {
            replaced = false;
            theSelection = sampleText;
        }

        txtarea.focus();

        // This has changed
        var text = theSelection;
        if( theSelection.charAt(theSelection.length-1) == " " )  // exclude ending space char, if any
        {
            theSelection = theSelection.substring( 0, theSelection.length - 1 );
            r = document.selection.createRange();
            r.text = tagOpen + theSelection + tagClose + " ";
        }
        else
        {
            r = document.selection.createRange();
            r.text = tagOpen + theSelection + tagClose;
        }

        if( ! replaced )
        {
            r.moveStart( 'character', -text.length - tagClose.length );
            r.moveEnd( 'character', -tagClose.length );
        }

        r.select();
    }
    else if( txtarea.selectionStart || txtarea.selectionStart == '0' )  // Mozilla
    {
        var startPos = txtarea.selectionStart;
        var endPos   = txtarea.selectionEnd;
        var replaced = ( endPos - startPos )

        var scrollTop = txtarea.scrollTop;
        var myText = txtarea.value.substring( startPos, endPos )  ||  sampleText;

        if( myText.charAt(myText.length-1) == " " )  // exclude ending space char, if any
            subst = tagOpen + myText.substring( 0, myText.length - 1 ) + tagClose + " ";
        else
            subst = tagOpen + myText + tagClose;

        txtarea.value = textbar_insertString( txtarea.value, startPos, endPos, subst );

        txtarea.focus();

        // set new selection
        if( replaced )
        {
            txtarea.selectionStart = txtarea.selectionEnd =
                startPos + tagOpen.length + myText.length + tagClose.length;
        }
        else
        {
            txtarea.selectionStart = startPos + tagOpen.length;
            txtarea.selectionEnd = txtarea.selectionStart + myText.length;
            txtarea.scrollTop = scrollTop;
        }
    }
    else  // All other browsers
    {
        var copy_alertText = alertText;
        var re1 = new RegExp( "\\$1", "g" );
        var re2 = new RegExp( "\\$2", "g" );
        copy_alertText = copy_alertText.replace( re1, sampleText );
        copy_alertText = copy_alertText.replace( re2, tagOpen + sampleText + tagClose );

        var text = ( sampleText ? prompt(copy_alertText) : '' ) || sampleText;
        text = tagOpen + text + tagClose;

        txtarea.value += "\n" + text;

        // in Safari this causes scrolling
        if( ! textbar_is_safari )
            txtarea.focus();
    }

    // reposition cursor if possible
    if( txtarea.createTextRange )
        txtarea.caretPos = document.selection.createRange().duplicate();
}


/**
 * Insert the selected filename and close the window
 *
 * @see http://www.alexking.org/index.php?content=software/javascript/content.php
 */
/*
function textbar_mediaSelect( file )
{
    textbar_insertText( opener.document.editform.wikitext, '{{' + file + '}}' );
    window.close();
}
*/

/*
 * Insert the given value at the current cursor position
 *
 * @see http://www.alexking.org/index.php?content=software/javascript/content.php
 */
function textbar_insertText( id, value )
{
    var field = document.getElementById(id);

    if( document.selection )  // IE
    {
        field.focus();

        ( opener ? opener.document : document ).selection.createRange().text = value;
    }
    else if( field.selectionStart || field.selectionStart == '0' )  // MOZILLA/NETSCAPE
    {
        var startPos = field.selectionStart;
        var scrollTop = field.scrollTop;
        field.value = textbar_insertString( field.value, startPos, field.selectionEnd, value );

        field.focus();

        field.selectionStart = field.selectionEnd = startPos + value.length;
        field.scrollTop = scrollTop;
    }
    else
    {
        field.value += "\n" + value;
    }

    // reposition cursor if possible
    if( field.createTextRange )
        field.caretPos = document.selection.createRange().duplicate();
}

/**
 * Replace a specified range in a target string with a new string.
 */
function textbar_insertString( target, iStart, iEnd, insert )
{
    return target.substring( 0, iStart ) + insert + target.substring( iEnd, target.length );
}


/**
 * Given a tagsets object as used in textbar_getInputFormat, return the current selected
 * input format as a string (one of the tagset property names), or undefined if not found
 */
function textbar_getInputFormatName( tagsets )
{
    var elements = document.getElementsByName( 'edit[format]' );
    for( var i = 0;  i < elements.length;  i++ )
    {
        var element = elements[i];
        if( element.checked )
        {
            var html = element.parentNode.innerHTML;

            for( name in tagsets )
            {
                if( html.search(name) >= 0 )
                    return name;
            }
        }
    }
}

/**
 * Get the current selected input format and return an object representing it:
 * { name: nameString, tagset: tagsetObject }
 */
function textbar_getInputFormat()
{
    var tagsets =
    {
        Markdown:
        {
            bold:    [ '**', '**' ],
            italic:  [ '*', '*' ],
            code:    [ '`', '`' ],
            fonth1:  [ '# ', '' ],
            fonth2:  [ '## ', '' ],
            fonth3:  [ '### ', '' ],
            fonth4:  [ '#### ', '' ],
            fonth5:  [ '##### ', '' ],
            link:    [ '[', '](page-name "tooltip")' ],
            extlink: [ '[', '](http://www.example.com/ "tooltip")' ],
            list:    [ '1. ', '\n' ],
            list_ul: [ '* ', '\n' ],
            rule:    [ '----\n' ]
        },

        Textile:
        {
            bold:    [ '*', '*' ],
            italic:  [ '_', '_' ],
            code:    [ '@', '@' ],
            fonth1:  [ 'h1. ', '' ],
            fonth2:  [ 'h2. ', '' ],
            fonth3:  [ 'h3. ', '' ],
            fonth4:  [ 'h4. ', '' ],
            fonth5:  [ 'h5. ', '' ],
            link:    [ '"', '":page-name' ],
            extlink: [ '"', '":http://www.example.com/' ],
            list:    [ '# ', '\n' ],
            list_ul: [ '* ', '\n' ],
            rule:    [ '<hr />\n' ]
        }
    }

    var format = { name: textbar_getInputFormatName( tagsets ) };
    if( format.name )
        format.tagset = tagsets[format.name];

    return format;
}
